#define ASLCXP_VERSION_STRING "3.2.0.0"
#define ASLCXP_VERSION 0x03020000
#define ASLCXP_MAX_DEVICES 64
#define ASLCXP_DEVICE_NAME "aslcxp"

typedef enum  cxp_protocol_version {
   cxp_unknown,
   cxp_1_0,
   cxp_1_1,
   cxp_2_0
} cxp_protocol_version_t;

typedef enum cxp_transacion_state {
   cxp_idle,
   cxp_started,
   cxp_tentative_ack,
   cxp_wait_ack,
   cxp_complete,
   cxp_error
} cxp_transaction_state_t;
 
struct cxp_transaction;

typedef struct cxp_ack_timer {
   struct timer_list        ack_timer_list;
   void                   (*callback)(struct cxp_transaction *);
} cxp_ack_timer_t;

typedef struct cxp_transaction {
   struct aslcxp_device   * cxp_device;
   wait_queue_head_t        wait_queue_head;
   struct cxp_ack_timer     ack_timer;
   MDA_CxpDataHeader        cxpheader;
   MDA_CxpData *            cxpdata;
   cxp_transaction_state_t  cxp_transaction_state;
   unsigned int             ioctl;
   long                     error_code;
   uint32_t                 protocol_error_code;
   uint32_t                 ack_timeout_ms;
} cxp_transaction_t;

struct aslcxp_device {
   struct module          * owner;
   struct mda_device      * mdadev;
   struct device          * device;
   struct cdev              cdev;
   dev_t                    major;
   dev_t                    minor;

   int                      open_count;
   struct file *            file_write_exclusive;
   struct mutex             open_mutex;

   spinlock_t               interrupt_spinlock;
   uint32_t                 shadow_interrupt_control;

   struct mutex             cxp_serialisation_mutex;
   cxp_protocol_version_t   cxp_protocol_version;

   cxp_transaction_t        cxp_transaction;
   uint32_t                 cxp_last_protocol_error;

   atomic_t                 log_count;

};

#define  to_mda_cxp(dev) container_of(dev, struct aslcxp_device, device)

irqreturn_t aslcxp_isr(struct mda_device *cxp_device, void *data);

int aslcxp_hw_start_op(struct aslcxp_device *);
void aslcxp_hw_start(struct aslcxp_device *);
void aslcxp_hw_reset(struct aslcxp_device *);

void aslcxp_init_ack_timer(struct cxp_ack_timer *, void (*callback)(struct cxp_transaction *));
void aslcxp_start_ack_timer(struct cxp_ack_timer *, unsigned int);
void aslcxp_restart_ack_timer(struct cxp_ack_timer *, unsigned int);
void aslcxp_stop_ack_timer(struct cxp_ack_timer *);
void aslcxp_delete_ack_timer(struct cxp_ack_timer *, unsigned int);
void aslcxp_timeout_callback(struct cxp_transaction *);

int aslcxp_complete_transaction_locked(struct cxp_transaction *, long , uint32_t );
void aslcxp_hw_restart_locked(struct aslcxp_device *);
void aslcxp_hw_reset_locked(struct aslcxp_device *);
void aslcxp_hw_start_locked(struct aslcxp_device *);
void aslcxp_hw_panic_locked(struct aslcxp_device *cxp_device);

uint32_t aslcxp_do_read32(struct aslcxp_device *cxp_device, uint32_t offset, const char *szWhence);
void aslcxp_do_write32(struct aslcxp_device *cxp_device, uint32_t offset, uint32_t value, const char *szWhence);

#define aslcxp_read32(d, o) aslcxp_do_read32((d), (o), __FUNCTION__)
#define aslcxp_write32(d, o, v) aslcxp_do_write32((d), (o), (v),  __FUNCTION__)

