#include "precomp.h"
#include <linux/time.h>

static __inline char *mda_get_address(struct mda_device *mda_device, uint32_t offset)
{
   if ((size_t)offset >= mda_device->device_address.kvsize) return NULL;
   else return mda_device->device_address.kvaddr + offset;
}

void mda_write32(struct mda_device *mda_device, uint32_t offset, uint32_t data)
{
   char *address = mda_get_address(mda_device, offset);
   if (address) __raw_writel(data, address);
   return;
}

void mda_write16(struct mda_device *mda_device, uint32_t offset, uint16_t data)
{
   char *address = mda_get_address(mda_device, offset);
   if (address) __raw_writew(data, address);
   return;
}

void mda_write8(struct mda_device *mda_device, uint32_t offset, uint8_t data)
{
   char *address = mda_get_address(mda_device, offset);
   if (address) __raw_writeb(data, address);
   return;
}

uint32_t mda_read32(struct mda_device *mda_device, uint32_t offset)
{
   char *address = mda_get_address(mda_device, offset);
   if (address) return __raw_readl(address);
   else return MDA_INVALID_READ;
}

uint16_t mda_read16(struct mda_device *mda_device, uint32_t offset)
{
   char *address = mda_get_address(mda_device, offset);
   if (address) return __raw_readw(address);
   else return 0xFFFF;
}

uint8_t mda_read8(struct mda_device *mda_device, uint32_t offset)
{
   char *address = mda_get_address(mda_device, offset);
   if (address) return __raw_readb(address);
   else return 0xFF;
}

uint64_t mda_gettime(void)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(4,0,0)
   struct timespec ts;

   ktime_get_ts(&ts);
#else
   struct timespec64 ts;

   ktime_get_ts64(&ts);
#endif
   return (((uint64_t)NSEC_PER_SEC * (uint64_t)ts.tv_sec)) + (uint64_t)ts.tv_nsec;
}

EXPORT_SYMBOL(mda_write32);
EXPORT_SYMBOL(mda_write16);
EXPORT_SYMBOL(mda_write8);
EXPORT_SYMBOL(mda_read32);
EXPORT_SYMBOL(mda_read16);
EXPORT_SYMBOL(mda_read8);
EXPORT_SYMBOL(mda_gettime);

